from enum import Enum

class Conteudo:
    nome = ""
    conteudoId = ""
    pjtId = ""
    items = []
    fullPath = []
    tipo = ""
    customAtts = []
    def __init__(self, Nome, contID, pjtID):
        self.nome = Nome
        self.conteudoId = contID
        self.pjtId = pjtID
        self.items = []
        self.fullPath = []


class Tipo(Enum):
    PASTA = 1
    ARQUIVO = 2


class Atividade:
    data = ""
    generator = ""
    agente_nome = ""
    agente_email = ""
    agente_tipo = ""
    verb = ""
    objeto_displayName = ""
    objeto_id = ""
    objeto_tipo = ""
    projeto_id = ""
    projeto_nome = ""
    def __init__(self, Data, gerador, AgntNome, Verbo, ObjNome, ObjId, PjtId, PjtNome):
        self.data = Data
        self.generator = gerador
        self.agente_nome = AgntNome
        self.verb = Verbo
        self.objeto_displayName = ObjNome
        self.objeto_id = ObjId
        self.projeto_id = PjtId
        self.projeto_nome = PjtNome


class CustomAttribute:
    fullpath = []
    name = ""
    value = ""
    def __init__(self, Nome, Valor):
        self.name = Nome
        self.value = Valor


class MapHub:
    hubName = ""
    hubId = ""
    projetos = []
    def __init__(self, hubNome, hubid):
        self.hubName = hubNome
        self.hubId = hubid


class Projeto:
    nome = ""
    pjtId = ""
    hubId = ""
    city = ""
    state_or_province = ""
    status = ""
    country = ""
    start_date = ""
    end_date = ""
    project_type = ""
    construction_type = ""
    Items = []
    container = ""
    def __init__(self, name, pjtID, hubID, container):
        self.Nome = name
        self.pjtId = pjtID
        self.hubId = hubID
        self.container = container
        self.Items = []


class Token:
    access_token = ""
    token_type = ""
    expires_in = 0
    date_origin = None
    def __init__(self, token, tipo, validade, tempo):
        self.access_token = token
        self.token_type = tipo
        self.expires_in = validade
        self.date_origin = tempo


class User:
    userId = ""
    nome = ""
    email = ""
    role = ""
    roleId = ""
    doc_man_access = ""
    insight_access = ""
    companyId = ""
    project_admin_access = ""
    def __init__(self, userID, Email, Nome, Role, RoleID, Company):
        self.userId = userID
        self.email = Email
        self.role = Role
        self.roleId = RoleID
        self.nome = Nome
        self.companyId = Company

class Company:
    companyId = ""
    nome = ""
    city = ""
    state_or_province = ""
    country = ""
    def __init__(self, CompanyId, Nome, Cidade, Estado, Pais):
        self.nome = Nome
        self.companyId = CompanyId
        self.city = Cidade
        self.state_or_province = Estado
        self.country = Pais


class Issue:
    created_at = ""
    updated_at = ""
    closed_at = ""
    closed_by = ""
    created_by = ""
    opened_at = ""
    opened_by = ""
    updated_by = ""
    title = ""
    description = ""
    due_date = ""
    due_date = ""
    status = ""
    assigned_to = ""
    assigned_to_type = ""
    created_at = ""
    created_by = ""


import DATAMANAGEMENT
import CLASSES

def DividirEmSubListas(lista, maximo):
    retorno = []
    imediato = []
    for elemento in lista:
        if len(imediato) < maximo:
            imediato.append(elemento)
        else:
            apend = imediato
            retorno.append(apend)
            imediato = [elemento]
    if len(imediato)>0:
        retorno.append(imediato)
    return retorno

def ListToJsonString(lista):
    return str("[\""+"\",\"".join(lista)+"\"]")

#Aqui deve ser passado o objeto Pasta
def Subpastas(Token, Pasta):
    items = DATAMANAGEMENT.GetFolderContents(Token, Pasta)
    for item in items:
        if item.tipo == CLASSES.Tipo.PASTA:  
            Subpastas(Token, item)
        else:
            pass



import requests
import BIM360
import CLASSES
import DATAMANAGEMENT
import Utils
from datetime import date


def getToken(client_id, client_secret, credentials):
    """Obtain Forge token given a client id & secret"""
    req = { 'client_id' : client_id, 'client_secret': client_secret, 'grant_type' : 'client_credentials','scope': credentials}
    resp = requests.post('https://developer.api.autodesk.com/authentication/v1/authenticate', req).json()
    #return resp['token_type'] + " " + resp['access_token']
    #return resp['access_token']
    token = CLASSES.Token(resp['access_token'], resp['token_type'], resp['expires_in'], date.today())
    return token


def getAccountUsers(token, hubId):
    url = "https://developer.api.autodesk.com/hq/v1/accounts/{0}/users".format(hubId)

    payload = ""
    headers = {
    'authorization': "Bearer {0}".format(token)
    }

    response = requests.request("GET", url, data=payload, headers=headers)
    
    usuarios = []
    
    for user in response.json():
        usuario = CLASSES.User(user['id'], user['email'], user['name'], "NA", "NA", "NA")
        usuarios.append(usuario)
    
    return usuarios

def getProjectUsers(token, pjtId, offset):
    url = "https://developer.api.autodesk.com/bim360/admin/v1/projects/{0}/users".format(pjtId)

    querystring = {"limit":"200", "offset":str(offset)}


    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers, params=querystring)
    
    usuarios = []
    
    for user in response.json()['results']:
        usuario = CLASSES.User(user['id'], user['email'],user['name'], user['jobTitle'], user['roleIds'], user['companyId'])
        usuarios.append(usuario)
        
    return [[response.json()['pagination']['limit'], response.json()['pagination']['offset'], response.json()['pagination']['totalResults']], usuarios]

def getBIM360Projects(token, hubId, offset):
    url = "https://developer.api.autodesk.com/hq/v1/accounts/{0}/projects".format(hubId)

    querystring = {"limit":"100","sort":"name", "offset":str(offset)}

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers, params=querystring)
    
    projetos = []
    
    for project in response.json():
        projeto = CLASSES.Projeto(project['name'], project['id'], hubId, "")
        projeto.city = project['city']
        projeto.construction_type = project['construction_type']
        projeto.country = project['country']
        projeto.end_date = project['end_date']
        projeto.start_date = project['start_date']
        projeto.state_or_province = project['state_or_province']
        projeto.project_type = project['project_type']
        projeto.status = project['status']
        projetos.append(projeto)
    
    return projetos

def getCustomAttribute(token, urnas, pjtId):
    url = "https://developer.api.autodesk.com/bim360/docs/v1/projects/{0}/versions:batch-get".format(pjtId)

    #payload = "{\"urns\": [\"urn:adsk.wipprod:dm.lineage:XLhFbCdgQvKG6x9W6oA5fg\",\"urn:adsk.wipprod:dm.lineage:OhEGeSv8SFOkvK-bFQZEQQ\"]}"
    payload = "{\"urns\": "+Utils.ListToJsonString(urnas)+"}"
    headers = {
    'content-type': "application/json",
    'authorization': "Bearer "+token
    }

    response = requests.request("POST", url, data=payload, headers=headers)
    
    atributos = []
    
    for attributes in response.json()['results']:
        for attribute in attributes['customAttributes']:
            atributo = CLASSES.CustomAttribute(attribute['name'], attribute['value'])
            atributos.append(atributo)
    
    return atributos

def getReviewActivity(token, pjtId):
    url = "https://developer.api.autodesk.com/bim360/admin/v1/projects/{0}/activities".format(pjtId)

    querystring = {"limit":"100"}

    headers = {
    'accept-language': "pt-BR",
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers, params=querystring)
    
    atividades = []
    
    for atividade in response.json()["streamItems"]:
        if atividade['activity']['verb']=="initiate-review-process" or atividade['activity']['verb']=="claim-review-task" or atividade['activity']['verb']=="submit-review":
            novaAtividade = CLASSES.Atividade(atividade['activity']['published'], atividade['activity']['generator'], atividade['activity']['actor']['displayName'], atividade['activity']['verb'], atividade['activity']['object']['displayName'], atividade['activity']['object']['id'], atividade['activity']['object']['project']['id'], atividade['activity']['object']['project']['displayName'])
            atividades.append(novaAtividade)
            print("nova atividade "+novaAtividade.objeto_displayName)
    
    
    return [response.json()['nextToken'], atividades]

def getReviewActivityToken(token, pjtId, pageToken):
    url = "https://developer.api.autodesk.com/bim360/admin/v1/projects/{0}/activities".format(pjtId)

    querystring = {"limit":"100","token":pageToken}

    headers = {
    'accept-language': "pt-BR",
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers, params=querystring)
    
    atividades = []
    
    for atividade in response.json()["streamItems"]:
        if atividade['activity']['verb']=="initiate-review-process" or atividade['activity']['verb']=="claim-review-task" or atividade['activity']['verb']=="submit-review":
            novaAtividade = CLASSES.Atividade(atividade['activity']['published'], atividade['activity']['generator'], atividade['activity']['actor']['displayName'], atividade['activity']['verb'], atividade['activity']['object']['displayName'], atividade['activity']['object']['id'], atividade['activity']['object']['project']['id'], atividade['activity']['object']['project']['displayName'])
            atividades.append(novaAtividade)
    
    nextToken = ""
    try:
        nextToken = response.json()['nextToken']
    except:
        pass
    
    return [nextToken, atividades]

def getProjectIssues(token, container):
    url = "https://developer.api.autodesk.com/issues/v1/containers/{0}/quality-issues".format(container)

    querystring = {"page[limit]":"100"}

    headers = {
    'authorization': "Bearer "+token
    }

    issues = []

    response = requests.request("GET", url, headers=headers, params=querystring)
    for issue in response.json()['data']:
        issue_obj = CLASSES.Issue()
        issue_obj.created_at = issue['attributes']['created_at']
        issue_obj.closed_at = issue['attributes']['closed_at']
        issue_obj.closed_by = issue['attributes']['closed_by']
        issue_obj.created_by = issue['attributes']['created_by']
        issue_obj.opened_at = issue['attributes']['opened_at']
        issue_obj.opened_by = issue['attributes']['opened_by']
        issue_obj.updated_by = issue['attributes']['updated_by']
        issue_obj.title = issue['attributes']['title']
        issue_obj.description = issue['attributes']['description']
        issue_obj.due_date = issue['attributes']['due_date']
        issue_obj.status = issue['attributes']['status']
        issue_obj.assigned_to = issue['attributes']['assigned_to']
        issue_obj.assigned_to_type = issue['attributes']['assigned_to_type']
        issue_obj.updated_at = issue['attributes']['updated_at']

        issues.append(issue_obj)
    
    return issues

def GetCompanies(token, hubId):
    url = "https://developer.api.autodesk.com/hq/v1/accounts/{0}/companies".format(hubId)

    querystring = {"limit":"100"}

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers, params=querystring)

    empresas = []

    for company in response.json():
        empresa = CLASSES.Company(company['id'], company['name'], company['city'], company['state_or_province'], company['country'])
        empresas.append(empresa)
    
    return empresas


import requests
import CLASSES
import re


def GetHubs(token):
    url = "https://developer.api.autodesk.com/project/v1/hubs"

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    hubs = []
    for hub in response.json()['data']:
        newHub = CLASSES.MapHub(hub['attributes']['name'], hub['id'])
        hubs.append(newHub)
    
    return hubs


def GetProjects(token, hubId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/{0}/projects".format(hubId)

    headers = {
    'authorization': "Bearer " + token
    }

    response = requests.request("GET", url, headers=headers)
    projetos = []
    for pjt in response.json()['data']:
        newPjt = CLASSES.Projeto(pjt['attributes']['name'], pjt['id'], hubId, pjt['relationships']['issues']['data']['id'])
        projetos.append(newPjt)
    
    return projetos

def GetProjectTopFolders(token, hubId, pjtId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/{0}/projects/{1}/topFolders".format(hubId, pjtId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    topfolders = []
    
    for folder in response.json()['data']:
        if re.match("Project Files", folder['attributes']['name']) or re.match("Arquivos do projeto", folder['attributes']['name']):
            topfolder = CLASSES.Conteudo(folder['attributes']['name'], folder['id'], pjtId)
            topfolder.tipo = CLASSES.Tipo.PASTA
            topfolders.append(topfolder)
    
    return topfolders

def GetFolderContents(token, Folder):
    url = "https://developer.api.autodesk.com/data/v1/projects/{0}/folders/{1}/contents".format(Folder.pjtId, Folder.conteudoId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    for content in response.json()['data']:
        try:
            conteudo = CLASSES.Conteudo("Noname", content['id'], Folder.pjtId)
            if content['type'] == 'folders':
                conteudo.nome = content['attributes']['name']
                conteudo.tipo = Conteudo.Tipo.PASTA
            else:
                conteudo.nome = content['attributes']['displayName']
                conteudo.tipo = Conteudo.Tipo.ARQUIVO
            Folder.items.append(conteudo)
        except:
            pass
    
    return Folder.items

def GetIssuesContainer(token, hubId, pjtId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/b.{0}/projects/b.{1}".format(hubId, pjtId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    return response.json()['data']['relationships']['issues']['data']['id']

def GetSubmittalsContainer(token, hubId, pjtId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/b.{0}/projects/b.{1}".format(hubId, pjtId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    return response.json()['data']['relationships']['submittals']['data']['id']

def GetRfisContainer(token, hubId, pjtId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/b.{0}/projects/b.{1}".format(hubId, pjtId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    return response.json()['data']['relationships']['rfis']['data']['id']

def GetMarkupsContainer(token, hubId, pjtId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/b.{0}/projects/b.{1}".format(hubId, pjtId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    return response.json()['data']['relationships']['markups']['data']['id']

def GetChecklistsContainer(token, hubId, pjtId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/b.{0}/projects/b.{1}".format(hubId, pjtId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    return response.json()['data']['relationships']['checklists']['data']['id']

def GetCostContainer(token, hubId, pjtId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/b.{0}/projects/b.{1}".format(hubId, pjtId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    return response.json()['data']['relationships']['cost']['data']['id']

def GetLocationsContainer(token, hubId, pjtId):
    url = "https://developer.api.autodesk.com/project/v1/hubs/b.{0}/projects/b.{1}".format(hubId, pjtId)

    headers = {
    'authorization': "Bearer "+token
    }

    response = requests.request("GET", url, headers=headers)
    
    return response.json()['data']['relationships']['locations']['data']['id']


from setuptools import setup, find_packages
 
classifiers = [
  'Development Status :: 4 - Beta',
  'Intended Audience :: Developers',
  'Operating System :: Microsoft :: Windows :: Windows 10',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3',
  'Natural Language :: Portuguese (Brazilian)',
  'Topic :: Software Development :: Libraries'
]

setup(
  name='FMOARPG',
  version='0.0.6',
  description='Biblioteca para integrar Forge e Power BI',
  long_description='Esta biblioteca foi desenvolvida para integrar as APIs da plataforma Forge com o Power BI de maneira direta e simplificada',
  url='https://github.com/Jpornelas/MAPyForg/tree/master/MAPyForg',
  author='John Martins',
  author_email='jpornelas@poli.ufrj.br',
  license='MIT',
  classifiers=classifiers,
  keywords='Forge',
  packages=find_packages(),
  install_requires=['']
)